class ManagedData {
    constructor(data) {
        this.data = data;
        this.history = [];
        this.historyPosition = 0;

        this.storeCurrentData("Initial state");
    }

    onUndo() {}
    onRedo() {}

    undo() {
        if (this.historyPosition + 1 >= this.history.length) {
            return false;
        }

        this.historyPosition += 1;
        this.data = JSON.parse(this.history[this.historyPosition].data);

        this.onUndo();

        return true;
    }

    redo() {
        if (this.historyPosition <= 0) {
            return false;
        }

        this.historyPosition -= 1;
        this.data = JSON.parse(this.history[this.historyPosition].data);

        this.onRedo();

        return true;
    }

    formatData(data) {
        return data;
    }

    storeCurrentData(description) {
        var formattedData = this.formatData(this.data);

        this.history.unshift({
            description: description,
            data: JSON.stringify(formattedData), // Creating a deep copy
        });

        // Forget about the currently stored potential future
        this.history.splice(0, this.historyPosition);
        this.historyPosition = 0;
    }
}
